% Ldrum.m  Compute eigenvalues of Laplacian on L-shaped region
%          T. Betcke and L. N. Trefethen 9/03
%
% The first three eigenvalues are computed by the method of
% particular solutions (Betcke & Trefethen, SIAM Review 2005).

% Compute subspace angles for various values of lambda:
  N = 36; k = 1:N;                        % orders in Bessel expansion
  np = 2*N;                               % no. of bndry & interior pts
  t1 = 1.5*pi*(.5:np-.5)'/np;             % angles of bndry pts
  r1 = 1./max(abs(sin(t1)),abs(cos(t1))); % radii of bndry pts
  t2 = 1.5*pi*rand(np,1);                 % angles of interior pts
  r2 = rand(np,1)./max(...
           abs(sin(t2)),abs(cos(t2)));    % radii of interior pts
  t = [t1;t2]; r = [r1;r2];               % bndry and interior combined
  lamvec = .2:.2:25; S = [];              % trial values of lam
  for lam = lamvec
    A = sin(2*t*k/3).*...
          besselj(2*k/3,sqrt(lam)*r);
    [Q,R] = qr(A,0);
    s = min(svd(Q(1:np,:))); S = [S s];   % subspace angle for this lam
  end

% Convert to signed subspace angles:
  I = 1:length(lamvec);                   % all lam points
  J = I(2:end-1);                         % interior points
  J = J( S(J)<S(J-1) & S(J)<S(J+1) );     % local minima
  J = J + (S(J-1)>S(J+1));                % points where sign changes
  K = 0*I; K(J) = 1;
  S = S.*(-1).^cumsum(K);                 % introduce sign flips
  subplot(3,1,1)
  hold off, plot(lamvec,S), hold on       % plot signed angle function
  plot([0 max(lamvec)],[0 0],'-k')        % plot lam axis

% Find eigenvalues via 9th-order interpolation:
  for j = length(J):-1:1
    I = J(j)-5:J(j)+4;
    lam = polyval(polyfit(S(I)/norm(S(I)),lamvec(I),9),0);
    plot(lam*[1 1],[-1 1],'r')
    text(lam,.6,sprintf('%13.9f',lam),'color','r')
  end

% Plot the first eigenfunction:
  [X,Y] = meshgrid(-1:.05:1,-1:.05:1); Z = X(:)+i*Y(:);
  p = [0 1i -1+1i -1-1i 1-1i 1];
  [in on] = inpolygon(real(Z),imag(Z),real(p),imag(p));
  zB = Z(on); zI = Z(in&~on); z = [zB;zI]; t = mod(angle(z/i),2*pi);
  A = besselj(2*k/3,sqrt(lam)*abs(z)).*sin(2*t*k/3);
  [Q,R] = qr(A,0); [U,S,V] = svd(Q(1:length(zB),:));
  V = V(:,end); Q = Q*V; [t,I] = max(abs(Q)); Q = Q/Q(I);
  F = NaN*zeros(size(Z));
  F(in&~on) = Q(length(zB)+1:end); F(on) = Q(1:length(zB),:);
  F = reshape(F,length(X),length(Y)); subplot(3,1,2:3)
  surf(X,Y,F), view(-150,40), axis off, zlim([0 .7])
