% OScritical.m  Find critical R for Orr-Sommerfeld problem
%               L. N. Trefethen 1/05
%
% The context here is as in orr_sommerfeld.m: plane Poiseuille flow.
% This code finds the critical parameters R (Reynolds number) and
% alpha (streamwise Fourier parameter) for eigenvalue instability:
% the smallest value of R for which, for some alpha, the linearized
% Navier-Stokes operator has an imaginary eigenvalue.  It gets R
% correct to about 8 digits.

  function OScritical()
  R0 = 6000;
  opts = optimset('tolx',1e-7);
  R = fzero(@fun1,R0,opts);         % critical Reynolds number



% fun1.m  Given R, optimizes over alpha to maximize the
%         largest real part of an Orr-Sommerfeld eigenvalue 
%         (actually, to minimize the negative of that)
 
  function s = fun1(R)
  opts = optimset('tolx',1e-6);
  [alpha,s] = fminbnd(@fun2,0,2,opts,R);
  disp(sprintf('R = %10.5f    alpha = %8.6f    s = %14.11f',R,alpha,s))



% fun2.m  Given alpha and R, finds maximum real part of an
%         Orr-Sommerfeld eigenvalue (actually its negative) 
 
  function s = fun2(alpha,R)

% 2nd- and 4th-order differentiation matrices:
  N = 48;
  [D,x] = cheb(N); D2 = D^2; D2 = D2(2:N,2:N);
  S = diag([0; 1 ./(1-x(2:N).^2); 0]);
  D4 = (diag(1-x.^2)*D^4 - 8*diag(x)*D^3 - 12*D^2)*S;
  D4 = D4(2:N,2:N);

% Orr-Sommerfeld operators A,B and generalized eigenvalues:
  I = eye(N-1);
  A = (D4-2*alpha^2*D2+alpha^4*I)/R - 2i*alpha*I ...
      - 1i*alpha*diag(1-x(2:N).^2)*(D2-alpha^2*I);
  B = D2-alpha^2*I;
  s = -max(real(eig(A,B)));

