% barycentric.m  Barycentric interpolation in Chebyshev points
%                L. N. Trefethen 12/04
%
% A fast, stable, and exponentially accurate way to represent a smooth
% function f on [-1,1] is polynomial interpolation in Chebyshev points 
% evaluated by the barycentric formula.  This method was proposed by
% Salzer (Computer J. 1972), publicized by Berrut & Trefethen (SIAM
% Review 2004), proved stable by N. Higham (IMA J. Numer. Anal. 2004).
% It is the basis of Battles' "chebfun" software system, and theorems
% concerning convergence are given by Battles and Trefethen (SIAM J.
% Sci. Comp. 2004).  This code, adapted from Berrut & Trefethen,
% illustrates the robustness of the method by applying it to a nonsmooth
% function f.  The interpolant is computed to machine accuracy, though
% it mtaches f to only 2-3 digits since f is not smooth.

  fun = inline('(x<=0).*(.5-abs(x+.5)) + (x>0).*(.1*sin(10*pi*x))');

  N = 100;                      % number of interpolation pts
  x = cos(pi*(0:N)'/N);         % Chebyshev points of 2nd kind
  f = fun(x);                   % function values to be interpolated
  xx = linspace(-1,1,2000)';    % interpolant is evaluated at 2000 pts
  numer = zeros(size(xx));
  denom = numer; exact = numer;
  c = [1/2
       ones(N-1,1)              % Salzer's coefficients
       1/2].*(-1).^((0:N)');         
  warning off                   % turn off harmless divide-by-0 warnings
  for j = 1:N+1                 % loop over interpolation points
    xdiff = xx-x(j);
    temp = c(j)./xdiff;
    numer = numer + temp*f(j);
    denom = denom + temp; 
    exact(xdiff==0) = j;        % check for sampling at interpolation pt
  end
  ff = numer./denom;            % the barycentric formula
  jj = find(exact);
  ff(jj) = f(exact(jj));        % use exact values at interpolation pts
  warning on
  
  % Plot the result:
  hold off, plot(xx,ff,'-b','linewidth',1.4)               
  hold on, plot(x,f,'.k','markersize',15)
  ylim([-.15 .55]), grid on
  title(['barycentric interpolant in ' int2str(N) ' Chebyshev points'])
