% cheb_coeffs.m  Use FFT to compute Chebyshev coeffs of f(x)=exp(x)
%                L. N. Trefethen 1/05
%
% A smooth function f(x) on [-1,1] has a Chebyshev expansion
%
% f(x) = SUM_{k=0}^infty a_k T_k(x),
%
% a_k = (2/pi) INT_{-1}^1 (1-x^2)^(-1/2) f(x) T_k(x) dx,
%
% where T_k is the kth Chebyshev polynomial (except for k=0 the factor
% is 1/pi, not 2/pi); see e.g. Mason & Handscomb, Chebyshev Polynomials,
% 2003.  Like Taylor coeffs, Chebyshev coeffs can be computed by the FFT
% (Geddes, SIAM J. Numer. Anal. 1978).  If f is analytic in the closed
% ellipse with foci 1 and -1 whose semiaxis lengths add to R>1, then a_k =
% O(R^{-k}) and the error in its approx. by an N-point FFT is O(R^{-N}).

  format long
  f = inline('exp(x)');                        % e^x just for simplicity
  N = 14;                                      % usually one uses bigger N
  x = cos(pi*(0:2*N-1)'/N);                    % Chebyshev points (twice)
  g = real(fft(f(x)))/(2*N);                   % FFT of f evaluated at pts
  a = [g(1); g(2:N)+g(2*N:-1:N+2)];            % Chebyshev coeffs
  exact = [besseli(0,1) 2*besseli(1:N-1,1)]';  % analytic solution
  error = a - exact;                           % error
  disp(['       computed    '...               % table headings
        '         exact     '...
        '         error     '])
  disp([a exact error])                        % print results
