% cheb_vs_gauss.m  Compare Chebyshev and Gauss quadrature in [-1,1]
%                  L. N. Trefethen 1/05
%
% Chebyshev quadrature here means integration of the polynomial
% interpolant to f(x) sampled at Chebyshev points of the 2nd kind --
% i.e., Clenshaw-Curtis quadrature, implemented here with the FFT.
% Gauss quadrature means, as usual, integration of the interpolant
% in Legendre points.  Change the "%" characters to vary f.  The
% polynomial degree is N; the number of points is N+1.

  s = 'exp(-x.^2)';   exact = sqrt(pi)*erf(1); % entire
% s = '1./(1+x.^2)';  exact = pi/2;            % analytic
% s = 'abs(x.^3)';    exact = 1/2;             % C^2
% s = 'cos(10*x)';    exact = sin(10)/5;       % two-phase
  f = inline(s);

  cc = []; gg = []; NN = 1:40;                 % vectors of results
  for N = NN                                   % loop over various N

% Chebyshev quadrature:
    x = cos(pi*(0:2*N-1)'/N);                  % Chebyshev points (twice)
    g = real(fft(f(x)))/(2*N);                 % FFT of f evaluated at pts
    a = [g(1); g(2:N)+g(2*N:-1:N+2); g(N+1)];  % Chebyshev coeffs
    w = zeros(1,N+1);
    w(1:2:end) = 2./(1-(0:2:N).^2);            % weight vector
    I = w*a;                                   % integral as inner product
    cc = [cc; I];                              % store this result

% Gauss quadrature:
    beta = .5./sqrt(1-(2*(1:N)).^(-2));        % 3-term recurrence coeffs
    T = diag(beta,1) + diag(beta,-1);          % Jacobi matrix
    [V,D] = eig(T);                            % eigenvalue decomposition
    x = diag(D); [x,i] = sort(x);              % Legendre points
    w = 2*V(1,i).^2;                           % Gauss quadrature weights
    I = w*f(x);                                % integral as inner product
    gg = [gg; I];                              % store this result

  end
  hold off, semilogy(NN,abs(cc-exact),'.-r')   % plot Chebyshev error (red)
  hold on, semilogy(NN,abs(gg-exact),'b')      % plot Gauss error (blue)
  grid on, legend('Chebyshev','Gauss')
  xlabel('degree N'), ylabel('error'), title(s)
