% circular_drum.m - First 20 eigenmodes of Laplacian on unit disk
%                   L. N. Trefethen 8/03
%
% The problem of computing eigenmodes of a circular drum with zero
% boundary condition is classical; see e.g. Sec. 206 of Rayleigh's
% Theory of Sound.  The solutions can be expressed in terms of Bessel
% functions.  Here they are computed by a Fourier-Chebyshev spectral
% method in polar coordinates following Fornberg (SIAM J. Sci. Comp.
% 1965); see also Chap. 18 of Boyd, Chebyshev and Fourier Spectral
% Methods, 2001.  This code is adapted from p28.m of Trefethen,
% Spectral Methods in MATLAB, and calls the function cheb.m from that
% book.  All numbers are accurate to the digits printed but the plots
% of nodal lines nevertheless look rough because the grid is coarse.

% r coordinate, ranging from -1 to 1 (N must be odd):
  N = 27; [D,r] = cheb(N); N2 = (N-1)/2; D2 = D^2;
  D1 = D2(2:N2+1,2:N2+1); D2 = D2(2:N2+1,N:-1:N2+2);
  E1 =  D(2:N2+1,2:N2+1); E2 =  D(2:N2+1,N:-1:N2+2);

% t = theta coordinate, ranging from 0 to 2*pi (M must be even):
  M = 18; dt = 2*pi/M; t = dt*(1:M)'; M2 = M/2;
  D2t = toeplitz([-pi^2/(3*dt^2)-1/6 ...
               .5*(-1).^(2:M)./sin(dt*(1:M-1)/2).^2]);

% Laplacian in polar coords (chap. 11, Spectral Methods in MATLAB):
  R = diag(1./r(2:N2+1)); Z = zeros(M2); I = eye(M2);
  L = kron(D1+R*E1,eye(M))+ kron(D2+R*E2,[Z I;I Z])+ kron(R^2,D2t);

% Compute 20 eigenmodes:
  index = 1:20;
  [V,Lam] = eig(-L); Lam = diag(Lam);
  [Lam,ii] = sort(real(Lam)); ii = ii(index); V = real(V(:,ii));
  Lam = sqrt(Lam(index)/Lam(1));

% Plot them:
  [rr,tt] = meshgrid(r(1:N2+1),[0;t]); [xx,yy] = pol2cart(tt,rr);
  z = exp(1i*pi*(-100:100)/100);
  [ay,ax] = meshgrid(.72:-.24:0,0:.20:.80); 
  for i = index
    u = reshape(real(V(:,i)),M,N2);
    u = [zeros(M+1,1) u([M 1:M],:)]; u = u/norm(u(:),inf);
    subplot('position',[ax(i) ay(i) .20 .24]), plot(z)
    axis(1.25*[-1 1 -1 1 -1 1]), axis off, hold on
    view(0,90), colormap([0 0 0]), axis square
    contour3(xx,yy,u-1,[-1 -1]), plot3(real(z),imag(z),-abs(z))
    text(-.7,1.18,sprintf('%13.10f',Lam(i)),'fontsize',8)
  end
