
% cuberootA.m  Compute the cube root of a matrix by a Cauchy integral
%              L. N. Trefethen 8/03
%
% If f is an analytic function and A is a matrix, f(A) can be computed
% by the Cauchy integral
%
% f(A)  =  (2*pi*i)^(-1) INT_G (zI-A)^(-1) f(z) dz
%
% where G is a closed contour enclosing the eigenvalues of A.  Often
% exponential accuracy can be achieved by taking G to be a circle and
% approximating the integral by the trapezoid rule.  Here we demonstrate
% the method for f(A) = A^(1/3).  Other functions such as e^A can be
% handled similarly.  For many functions other methods such as Newton's
% method may be superior, but this approach is certainly effective.  For
% larger matrices it would speeds things up to do a preliminary Schur or
% Hessenberg reduction (see Golub & Van Loan, 3rd ed., Problem P7.4.3).

  format long, format loose
  N = 4;                               % dimension of A
  A = randn(N)/sqrt(N) + 3*eye(N)      % eigs approx in disk rad 1, ctr 3
  disp('    np       ||B^3 - A||')     % table headings
  for np = 2.^(1:6)                    % number of pts in trapezoid rule
    circle = exp(2i*pi*(1:np)/np);     % np pts on the unit circle
    z0 = 3; radius = 2;                % ctr, radius of circular contour
    z = z0 + radius*circle;            % nppoints on the contour
    I = eye(N); B = zeros(N);
    for i = 1:np
      R = inv(z(i)*I-A);               % resolvent matrix at point z(i)
      B = B + R*(z(i)-z0)*z(i)^(1/3);  % add up contributions to integral
    end
    B = real(B)/np;                    % clean up rounding errors
    resid = norm(B^3-A);               % compute the residual
    fprintf('%6d%20.14f\n',np,resid)   % display this result
  end
  disp(' ')
  disp('press <return> to see B')      % wait for user input
  pause, B                             % print final matrix
