% gamma_talbot.m  Compute complex gamma function via Talbot contour
%                 L. N. Trefethen 6/05
%
% Hankel's integral formula for Gamma(z) is
%
% 1/Gamma(z) = (2*pi*i)^(-1) int_C e^t t^{-z} dt
%
% where the contour C passes from -infty-0i around 0 to -infty+0i. 
% We evaluate the integral by the trapezoid rule transplanted to a
% cotangent contour due to Talbot (1979) with parameters optimized
% by Weideman (2005).  Adapted from p31.m of Trefethen, "Spectral
% Methods in MATLAB".

  N = 40;                                        % no. of quadrature pts
  th = (-N/2+.5:N/2-.5)*pi/(N/2);                % trapezoid pts in [-pi,pi]
  a = -.2407; b = .2387; c = .7409; d = .1349i;  % Weideman's parameters
  z = N*(a + b*th.*cot(c*th) + d*th);            % Talbot pts in z-plane
  zp = b*cot(c*th) - b*c*th./sin(c*th).^2 + d;   % N^(-1) times derivative
  x = -3.5:.1:4; y = -2.5:.1:2.5;                
  [xx,yy] = meshgrid(x,y);                     
  zz = xx + 1i*yy;                               % plotting grid
  gaminv = zeros(size(zz));
  Npts = length(th)                              % no. of pts on contour
  for k = 1:Npts                                 % Loop goes over contour
    t = z(k);                                    %   points, not grid pts
    gaminv = gaminv + exp(t)*t.^(-zz)*zp(k);     %   (which are vectorized)
  end
  gam = 1i./gaminv;                              % Gamma(z) on the grid
  figure, mesh(xx,yy,abs(gam))                   % plot |Gamma(z)|
  axis([-3.5 4 -2.5 2.5 0 6])
  xlabel Re(z), ylabel Im(z)
  text(4,-1.4,5.5,'|\Gamma(z)|','fontsize',20)
  colormap([0 0 0])                              % make the image black
  format long, disp(gam(26,46:10:76).')          % approxs. to Gamma(1:4)
