% isodrum.m - Third eigenmode of Laplacian on isospectral octagon 
%             L. N. Trefethen 4/05
%
% The eigenmode is computed to about 8 digits by the method of particular
% solutions.  This image first computed by Driscoll (SIAM Review 1999).
  
% Define geometry, fix boundary and interior pts:
  function isodrum()
  v = [0 2 2-2i 4 4+2i 2i 4i 2i-2];                  % vertices 
  N = 16;                                            % max Bessel order
  s = -cos((1:N)'*pi/(N+1)); s = .5*(s+1);           % N pts in [0,1]
  bd = v(8) + s*(v(1)-v(8));
  for j = 1:7, bd = [bd;v(j)+s*(v(j+1)-v(j))]; end   % boundary pts
  x = -2+6*rand(50*N,1); y = -2+6*rand(50*N,1);
  ii = inpolygon(x,y,real(v),imag(v));
  int = x(ii) + 1i*y(ii); int = int(1:10*N);         % interior points
  z = [bd; int];                                     % all pts
  Nbd = length(bd);                                  % no. of boundary pts

% Find eigenvalue and coeffs of eigenmode:
  opt = optimset('tolx',1e-8);
  lam = fminbnd(@isofun,5,6,opt,N,Nbd,z,v);          % find eig in [5,6]
  N = 12;                                            % small N for plotting
  [f,A,Q,R] = isofun(lam,N,Nbd,z,v);
  [U,S,V] = svd(Q(1:Nbd,:)); c = R\V(:,end);         % coeffs of eigenmode

% Plot eigenmode:
  [X,Y] = meshgrid(-2:.075:4,-2:.075:4);             % plotting grid
  z = X(:) + 1i*Y(:);
  [f,A] = isofun(lam,N,Nbd,z,v); F = A*c;            % construct eigenmode
  in = inpolygon(real(z),imag(z),real(v),imag(v));
  F(~in) = NaN; F = F/max(abs(F));                   % make exterior blank
  F = reshape(F,size(X)); hold off
  surf(X,Y,F,'Linestyle','none'), caxis([-1 1])      % color plot
  view(2), axis off, hold on
  contour3(X,Y,F,-.8:.2:.8), axis equal              % level curves
  set(get(gca,'children'),'edgecolor','k')           % make curves black
  plot(v([1:8 1]),'-k')                              % plot polygon
  text(.5,2.4,sprintf('%11.7f',lam),'fontsize',22)   % print eigenvalue


  function [f,A,Q,R] = isofun(lam,N,Nbd,z,v)
  w1 = z-v(1); t1 = angle(w1);      r1 = abs(w1);    % angles and radii
  w2 = z-v(2); t2 = angle(w2)+pi/2; r2 = abs(w2);    %   of sample points
  w4 = z-v(4); t4 = angle(w4/1i);   r4 = abs(w4);    %   with respect to
  w6 = z-v(6); t6 = angle(w6/1i);   r6 = abs(w6);    %   singular corners
  t6(t6<0) = t6(t6<0)+2*pi;                          % fix up t6 branch
  k = 1:N;                                           % Bessel orders
  a1 = 4*k/3; a2 = 2*k/3; a4 = 4*k/3; a6 = 2*k/3;    % scaled orders
  A = [sin(t1*a1).*besselj(a1,sqrt(lam)*r1)...       % matrix of 
       sin(t2*a2).*besselj(a2,sqrt(lam)*r2)...       %   particular solns
       sin(t4*a4).*besselj(a4,sqrt(lam)*r4)...
       sin(t6*a6).*besselj(a6,sqrt(lam)*r6)];
  [Q,R] = qr(A,0); f = min(svd(Q(1:Nbd,:)));         % subspace angle
