% jaisson.m  Solve a Laplace problem posed by Denis Jaisson
%            L. N. Trefethen 12/04
%
% Let I = [-1,1] and let a>1 and b>0 be constants.  We seek the function 
% u satisfying Laplace's equation in the region of the complex plane
% exterior to the four slits given by I plus the constants c2 = -a+ib,
% c1 = a+ib, d2 = -a-ib, d1 = a-ib, with u=1 on the c1 and d1 slits,
% u=-1 on the c2 and d2 slits, and u(inf) = 0.  We represent u by a sum
%
% u(z) =  Re { c(0)*[  log(jk(z-c1)) - log(jk(c2-z))
%                    + log(jk(z-d1)) - log(jk(d2-z)) ]
%
%      + SUM_{k=1}^N c(k)*[  jk(z-c1)^(-k) - jk(c2-z)^(-k) ]
%                          + jk(z-d1)^(-k) - jk(d2-z)^(-k) ] }
%
% where jk(z) = z+sqrt(z^2-1) maps exterior(I) to exterior(unit disk).
% The coefficients c(k) are found by linear least-squares.

  function jaisson()
  a = 2; b = 1;                             % define the geometry
  c1 = a+1i*b; c2 = -a+1i*b;                % centers of slits in upper...
  d1 = a-1i*b; d2 = -a-1i*b;;               %    ...and lower half-planes
  N = 18;                                   % no. of terms in expansion
  npts = 3*N;                               % no. of sample points
  circle = exp((1:npts)'*2i*pi/npts);       % unit circle
  circle = (1+1e-14)*circle;                % expand it slightly
  I = .5*(circle+1./circle);                % unit interval
  z = c1 + I;                               % sample points on slit
  A =  log(jk(z-c1))-log(jk(c2-z))...
     + log(jk(z-d1))-log(jk(d2-z));
  for k = 1:N                               % set up least-squares matrix
    A = [A (jk(z-c1).^(-k)-jk(c2-z).^(-k)...
           +jk(z-d1).^(-k)-jk(d2-z).^(-k))];
  end
  A = real(A);                              % symmetry is imposed here
  c = A\ones(size(z))                       % solve least-squares problem

% Contour plot of the solution:
  x = linspace(-5,5,85); y = linspace(-4,4,65);
  [xx,yy] = meshgrid(x,y); zz = xx+1i*yy;
  gg = c(1)*(log(jk(zz-c1))-log(jk(c2-zz))...
            +log(jk(zz-d1))-log(jk(d2-zz)));
  for k = 1:N
    gg = gg + c(k+1)*(jk(zz-c1).^(-k)-jk(c2-zz).^(-k)...
                    + jk(zz-d1).^(-k)-jk(d2-zz).^(-k));
  end
  gg = real(gg); levels = -1:.1:1; contour(xx,yy,gg,levels)
  hold on, axis equal, axis([-5 5 -4 4]), colormap([0 0 0])
  set(gca,'xtick',-4:2:4,'ytick',-4:2:4)
  plot(c1+I,'-b'), plot(c2+I,'-b'), plot(d1+I,'-b'), plot(d2+I,'-b')

  function jk = jk(z)       % Joukowski map ext(interval) -> ext(disk)
  main_branch = (real(z)>0)|((real(z)==0)&(imag(z)>0));
  sgn = 2*main_branch - 1; jk = z + sgn.*sqrt(z.^2-1);
