% kuramoto_siv.m  Solve Kuramoto-Sivashinsky eq. by ETDRK4 scheme
%                 A.-K. Kassam and L. N. Trefethen 4/03
%
% We consider the time-dependent PDE u_t = -u*u_x - u_xx - u_xxxx
% with periodic BCs on [0,32*pi].  The solutions are chaotic, with
% small perturbations growing by a factor of 10^8 over the range
% 0<t<150.  This code achieves 10-digit accuracy for small t and
% graphical accuracy for larger t.  The x discretization is Fourier
% spectral (see e.g. Trefethen, "Spectral Methods in MATLAB") and
% the t discretization is ETDRK4 4th-order (see Cox & Matthews, J.
% Comp. Phys. 2002 and Kassam & Trefethen, SIAM J. Sci. Comp. 2004).

% Spatial grid and initial condition:
  N = 128; x = 32*pi*(0:N-1)'/N;  
  u = cos(x/16).*(1+sin(x/16)); v = fft(u);

% Precompute constants by contour integrals (Kassam-Trefethen):
  h = 1/4;                          % time step
  k = [0:N/2-1 0 -N/2+1:-1]'/16;    % wave numbers
  L = k.^2 - k.^4;                  % Fourier multipliers
  E = exp(h*L); E2 = exp(h*L/2);
  M = 16;                           % no. of pts for complex means
  r = exp(1i*pi*((1:M)-.5)/M);      % roots of unity
  LR = h*L(:,ones(M,1)) + r(ones(N,1),:);
  Q  = h*real(mean(           (exp(LR/2)-1)./LR              ,2));
  f1 = h*real(mean(  (-4-LR+exp(LR).*(4-3*LR+LR.^2))./LR.^3  ,2));
  f2 = h*real(mean(    (4+2*LR+exp(LR).*(-4+2*LR))./LR.^3    ,2));
  f3 = h*real(mean(  (-4-3*LR-LR.^2+exp(LR).*(4-LR))./LR.^3  ,2));

% Time-stepping by ETDRK4 formula (Cox-Matthews):
  uu = u; tt = 0;
  tmax = 150; nmax = round(tmax/h); nplt = floor((tmax/100)/h);
  g = -0.5i*k;
  for n = 1:nmax
    t = n*h;
    Nv = g.*fft(real(ifft(v)).^2);
    a = E2.*v + Q.*Nv;
    Na = g.*fft(real(ifft(a)).^2);
    b = E2.*v + Q.*Na;
    Nb = g.*fft(real(ifft(b)).^2);
    c = E2.*a + Q.*(2*Nb-Nv);
    Nc = g.*fft(real(ifft(c)).^2);
    v = E.*v + Nv.*f1 + (Na+Nb).*f2 + Nc.*f3;
    if mod(n,nplt)==0
      u = real(ifft(v)); uu = [uu u]; tt = [tt t];
    end
  end
  
% Plot the solution:
  figure, surf(x,tt,uu'), shading interp, lighting phong, axis tight
  view([0 90]), colormap(autumn), zlim([-5 50])
  light('color',[1 1 0],'position',[1 2 2])
  material([.3 .6 .6 40 1])
