% many_disks.m  Compute Green's function exterior to several disks
%               L. N. Trefethen 8/03
%
% Suppose we seek the real function g(z) that is harmonic exterior to
% disks with centers c(j) and radii r(j) in the z-plane with g(z)~log|z|
% as |z|->infty.   This code is the analogue of two_disks.m for this
% more general problem.  The Green's function is approximated by series
% 
% g(z) = e + SUM_{j=1}^J
%
%     d(j)*log|z-c(j)| + Re[sum_{k=1}^N (a(j,k)-i*b(j,k))*(z-z(j))^-k]
%
% with SUM d(j) = 1; all these coeffs are collected in the vector X.
% The unknowns determined by linear least-squares are e, d(2),...,d(J),
% {a(j,k)}, {b(j,k)}.  This method was used by Finn et al. in "Topo-
% logical chaos in inviscid and viscous mixers", J. Fluid Mech. 2003.

% Solve the problem:
  J = 3; c = [-2 2+1i 1-2i]; r = [1 .5 .7];  % define the geometry
  N = 10;                                    % no. terms in expansions
  npts = 30;                                 % no. sample pts on circles
  circ = exp((1:npts)'*2i*pi/npts);          % roots of unity
  z = []; for j = 1:J
     z = [z; c(j)+r(j)*circ]; end            % collocation points
  A = ones(size(z));                         % the constant term
  for j = 1:J
    A = [A log(abs(z-c(j)))];                % the logarithmic terms
    for k = 1:N
      zck = (z-c(j)).^(-k);
      A = [A real(zck) imag(zck)];           % the algebraic terms
    end
  end
  X = -A(:,[1 3:end])\A(:,2);                % solve least-squares prob.
  X = [X(1); 1; X(2:end)]/...
         (1+sum(X(2*N+2:2*N+1:end)));
  e = X(1); X(1) =[];
  d = X(1:2*N+1:end); X(1:2*N+1:end) = [];
  a = X(1:2:end); b = X(2:2:end);

% Plot the result:
  x = linspace(-5,5,95); y = linspace(-4,4,75);
  [xx,yy] = meshgrid(x,y); zz = xx+1i*yy; gg = e*ones(size(zz));
  for j = 1:J
    gg = gg+d(j)*log(abs(zz-c(j)));
    for k = 1:N, zck = (zz-c(j)).^(-k); kk = k+(j-1)*N;
      gg = gg+a(kk)*real(zck)+b(kk)*imag(zck); end
  end
  for j = 1:J, gg(abs(zz-c(j))<r(j)) = 0; end
  levels = 1.1:.1:4; z = exp(pi*1i*(-50:50)'/50); hold off
  for j = 1:J, disk = c(j)+r(j)*z; fill(real(disk),imag(disk),[1 .7 .7])
    hold on, plot(disk,'-r','linewidth',2), end
  contour(xx,yy,gg,log(levels)), axis equal, axis([-5 5 -4 4])
  colormap([0 0 0]), set(gca,'xtick',-4:2:4,'ytick',-4:2:4)
