% mathieu.m  Mathieu eigenvalues following Abramowitz & Stegun
%            L. N. Trefethen 6/03
%
% The Mathieu operator is L: u -> -u_xx + 2qcos(2x)u with periodic
% boundary conditions on [-pi,pi], where q is a real parameter. 
% A plot of eigenvalues of L as a function of q is given on p. 724
% of Abramowitz & Stegun, Handbook of Mathematical Functions, 1964.
% This program produces almost the same image based on numbers 
% computed to close to machine precision by a Fourier spectral
% collocation method (small but interesting differences can be seen
% on close inspection).  The idea comes from Weideman and Reddy, ACM
% Trans. Math. Softw. 2001, and this particular code is adapted from
% p21.m of Trefethen, Spectral Methods in MATLAB.

  N = 42;                                        % no. of grid points
  h = 2*pi/N; x = h*(1:N);                       % grid
  D2 = toeplitz([-pi^2/(3*h^2)-1/6 ...           % differentiation matrix
      -.5*(-1).^(1:N-1)./sin(h*(1:N-1)/2).^2]);
  qq = 0:.2:15;                                  % vector of values of q
  data = [];
  for q = qq;                                    % for each value of q
    e = sort(eig(-D2 + 2*q*diag(cos(2*x))))';    %    compute eigenvalues
    data = [data; e(1:11)];                      %    and store them
  end
  figure, subplot(1,2,1)
  set(gca,'colororder',[0 0 1],...
          'linestyleorder','-|--')
  hold on, plot(qq,data)                         % draw the plot
  xlabel q, ylabel \lambda                       %    in panel 1
  axis([0 15 -24 32])
  set(gca,'ytick',-24:4:32)
  subplot(1,2,2)                                 % print a message
  c = 'color'; f = 'fontsize'; r = [.4 0 .7];    %    in panel 2
  text(0,.70,'This plot duplicates',c,r,f,15)
  text(0,.64,'Fig. 20.1 of Abramowitz',c,r,f,15)
  text(0,.58,'& Stegun (1964)',c,r,f,15)
  axis([0 1 0 1]), axis off
