
% orr_sommerfeld.m  Eigenvalues of Orr-Sommerfeld operator 
%                   L. N. Trefethen 8/03
%
% In classical hydrodynamic stability theory, stability of plane
% Poiseuille flow (between two infinite flat plates) is determined by
% looking for eigenvalues in the right half-plane of the Navier-Stokes
% operator linearized about the laminar flow solution (parabolic
% velocity profile).  Orszag showed that at Reynolds number R~=5772
% and Fourier parameter alpha~=1.02, an eigenvalue crosses into the
% right half-plane (J. Fluid Mech. 1971).  This code uses spectral
% methods to show these eigenvalues for four successively better
% resolutions.  The method is derived from Schmid & Henningson (1993)
% and Weideman & Reddy (ACM Trans. Math. Softw. 2001) and the code
% comes from p40.m of Trefethen, "Spectral Methods in MATLAB", 2000.

  R = 5772.2217;                              % Reynolds number
  a = 1.02055;                                % Fourier param. alpha
  [ay,ax] = meshgrid([.54 .04],[.1 .5]);      % plotting grid
  for N = 40:20:100                           % various resolutions

% 2nd- and 4th-order differentiation matrices:
    [D,x] = cheb(N); D2 = D^2; D2 = D2(2:N,2:N);
    S = diag([0; 1 ./(1-x(2:N).^2); 0]);
    D4 = (diag(1-x.^2)*D^4 - 8*diag(x)*D^3 - 12*D^2)*S;
    D4 = D4(2:N,2:N);

% Orr-Sommerfeld operators A,B and generalized eigenvalues:
    I = eye(N-1);
    A = (D4-2*a^2*D2+a^4*I)/R ...
         - 2i*a*I - 1i*a*diag(1-x(2:N).^2)*(D2-a^2*I);
    B = D2-a^2*I;
    e = eig(A,B);
    i = N/20-1; subplot('position',[ax(i) ay(i) .37 .37])
    plot(e,'.','markersize',12)
    grid on, axis([-.8 .2 -1 0]), axis square
    set(gca,'fontsize',7)
    title(['N = ' int2str(N) '    \lambda_{max} = ' ...
      num2str(max(real(e)),'%14.10f')],'fontsize',10), drawnow
  end
