% pade.m  Compute Pade coefficients of a function f
%         L. N. Trefethen 3/05
%
% Let f(z) be analytic in the closed disk of radius R>0 about z=0.
% This program uses the most straightforward of methods to compute
% the coefficients of the (m,n) Pade approximant of f, i.e., the
% rational function of type (m,n) whose Taylor series matches that
% of f as far as possible.  We assume the generic case where
% f - r = O(z^(m+n+1)).  Notation: r = p/q with
% f(z) = a(1) + a(2)z + a(3)z^2 + ...
% p(z) = b(1) + b(2)z + ... + b(m+1)z^m
% q(z) =   1  + c(1)z + ... + c( n )z^n

  f = inline('exp(z)');                   % function to approximate
  N = 64;                                 % no. of pts. in FFT
  R = 1;                                  % radius of circle
  zz = R*exp(2i*pi*(0:N-1)'/N); 
  a = fft(f(zz))/N;          
  a = real(a);                            % delete if f is complex
  a = a./R.^(0:length(a)-1)';             % Taylor coeffs. of f
  m = 5; n = 5;                           % degrees of numer & denom

% Find denominator coefficients:
  if n==0, b = a(1:m+1), break, end       % n=0 is a special case
  firstcol = [zeros(n-m-1,1)
              a(max([1,m-n+2]):m+1)];
  lastrow = a(m+1:m+n);
  H = hankel(firstcol,lastrow);
  rhs = -a(m+2:m+n+1);
  c = H\rhs;
  c = [1; flipud(c)];   
  disp('denominator coeffs:'), disp(c)
  z = roots(flipud(c));  
  disp('poles:'), format long, disp(z)                                

% Find numerator coefficients:
  if m <= n
    firstcol = [zeros(m,1); a(1)];
    lastrow = a(1:m+1);
    H = hankel(firstcol,lastrow);
    b = H*flipud(c(1:m+1));
  else
    firstcol = [zeros(n,1); a(1:m-n+1)];
    lastrow = a(m-n+1:m+1);
    H = hankel(firstcol,lastrow);
    b = H*flipud(c);
  end
  disp('numerator coeffs:'), disp(b)

% Evaluate at z = 1:
  z = 1;
  fprintf('                f at z = %3.1f: %15.11f\n', z, f(z))
  value = polyval(flipud(b),z)/polyval(flipud(c),z);
  fprintf('(%1d,%1d) Pade approx at z = %3.1f: %15.11f\n', m, n, z, value)
