% rayleigh_disks.m  Potential exterior to a rectangular array of disks
%                   L. N. Trefethen 1/05
%
% Lord Rayleigh (Phil. Mag. 1892) considered the problem of the 
% potential in the region exterior to an infinite rectangular array
% of circular disks.  He observed that the solution can be expressed
% by a rapidly convergent series.  This code computes that solution
% and plots it in a fundamental region.  We take the disks to have
% radius 1 and the rectangles to have half-length L and half-width W.
% The solutions are the real or imaginary parts of
%       w = SUM_{k=1}^N c(k)(z^s+z^(-s)) (s = 2*k-1).
% We find the coefficients c(k) by solving a least-squares problem, a
% discretization of the conditions Im w'=0 on the right boundary of T,
% Im w=1 on the top boundary.  Compare trapmap.m.

% Compute the coefficients:
  function rayleigh_disks()
  L = 2; W = 1.5;                           % dimensions of rectangle
  for N = 2:2:24                            % try various values of N
    np = 2*N;                               % no. of points on bndries
    z1 = L+linspace(0,1i*W,np).';           % points on right boundary
    z2 = 1i*W+linspace(0,L,np).';           % points on top boundary
    A1 = []; A2 = [];
    for k = 1:N
      s = 2*k-1;
      A1 = [A1 s*z1.^(s-1)-s*z1.^(-s-1)];   % w' on right boundary
      A2 = [A2 z2.^s+z2.^(-s)];             % w on top boundary
    end
    A = [imag(A1); imag(A2)];               % matrix of sampled fncts
    scl = (W^2+L^2).^(-1:-1:-N)';           % rescaling vector
    A = A*diag(sparse(scl));                % rescale the columns of A
    rhs = [zeros(np,1); ones(np,1)];        % right-hand side
    c = scl.*(A\rhs);                       % find coefficient vector
    s = 1:2:2*N; mu = (L.^s+L.^(-s))*c;     % conformal modulus
    fprintf('N = %2d  mu = %11.9f\n',N,mu)  % print modulus for this N
  end
  format short e, c                         % print coefficient vector

% Plot the map:
  f = figure;
  set(f,'defaultlinelinewidth',3)
  set(f,'defaultlinemarkersize',4)
  d = 0.08; [x,y] = meshgrid(-L:d:L,0:d:W);
  z = x+1i*y; z = z(abs(z)>1);
  draw(z,c,'.k')                            % points in domain
  draw(L+1i*linspace(-W,W),c,'r')           % left and right
  draw(1i*W+linspace(-L,L),c,'b')           % top and bottom 
  draw(exp(1i*linspace(0,pi,101)),c,'g')    % boundary of disk
  title(sprintf('modulus = %11.9f',mu))
  for i=1:2, subplot(2,1,i), axis equal
     axis(1.05*axis), axis off, end

  function draw(z,c,color)   % draw points/lines in domain and range
  w = 0*z;
  for k = 1:length(c), s = 2*k-1; w = w+c(k)*(z.^s+z.^(-s)); end
  subplot(2,1,1), plot(z,color), hold on, plot(-z,color)
  subplot(2,1,2), plot(w,color), hold on, plot(-w,color)
