% real_cf_approx  Caratheodory-Fejer approximation on [-1,1]
%                 L. N. Trefethen 1/05
%
% Best rational approximations to f(x) on [-1,1] equioscillate and
% can be computed by the Remes algorithm.  If f(x) is smooth, the
% simpler Caratheodory-Fejer method produces results that come
% extraordinarily close to equioscillation and optimality; see
% T. and Gutknecht, SIAM J. Numer. Anal. 1983.  This code is adapted
% from Trefethen, "Matlab codes for CF approximation", in Approximation
% Theory V, 1986 (one of the first papers ever to publish a MATLAB
% code).  The number s that it prints matches the best approximation
% error for exp(x) on [-1,1] to 10 digits.  For less smooth f, the
% approximations are less good and larger K is needed.

  f = inline('exp(x)');                    % function to approximate
  K = 16;                                  % no. of Cheby. coeffs
  m = 3; n = 3;                            % numer. and denom. degrees
  nfft = 256;                              % no. of points for FFT
  dim = K+n-m;                             % dimension of Hankel matrix
  np = n+1; nfft2 = nfft/2;                % convenient abbreviations
  z = exp(2*pi*1i*(0:nfft-1)/nfft);        % roots of unity
  x = real(z);                             % Chebyshev points (twice)
  fx = f(x); c = real(fft(fx))/nfft2;
  H = hankel(c(1+mod((1:dim)+m-n,nfft)));  % Hankel matrix
  [U,S,V] = svd(H);                        % SVD of H
  s = S(np,np); u = U(dim:-1:1,np)';       % the needed singular value
  v = V(:,np)';                            %     and vector
  zr = roots(v); qout = zr(abs(zr)>1);     % roots outside the disk
  qc = real(poly(qout)); qc = qc/qc(np);
  q = polyval(qc,z);
  Q = q.*conj(q); Qc = real(fft(Q))/nfft2; % denominator
  Qc(1) = Qc(1)/2; Q = Q/Qc(1);
  Qc = Qc(1:np)/Qc(1);
  b = fft([u zeros(1,nfft-dim)])./fft([v zeros(1,nfft-dim)]);
  Rt = fx-real(s*z.^K.*b);
  Rtc = real(fft(Rt))/nfft2;
  gam = real(fft((1)./Q))/nfft2;
  gam = toeplitz(gam(1:2*m+1));
  if m==0 Pc = 2*Rtc(1)/gam;
     else Pc = 2*[Rtc(m+1:-1:2) Rtc(1:m+1)]/gam; end
  Pc = Pc(m+1:2*m+1); Pc(1) = Pc(1)/2;
  P = real(polyval(Pc(m+1:-1:1),z));       % numerator
  R = P./Q;                                % rational CF approximant
  plot(x,fx-R,'-',x,[s;0;-s]*ones(1,nfft),'--')
  text(-.5,1.06*s,sprintf('s = %0.10g',s),'fontsize',18)
  s, error = norm(fx-R,'inf'), Pc, Qc
