% schwarz_christoffel.m  Schwarz-Christoffel map of a polygon
%                        L. N. Trefethen 2/05
%
% Let P be a polygonal region in the complex plane with N vertices
% w(k) and interior angles pi*a(k).  Any conformal map f of the unit
% disk to P can be represented by the S-C integral
%       f(z) = A + C INT_0^z PROD (1-z/z(k))^(a(k)-1) dz
% for some constants A and C and prevertices z(k) on the unit circle.
% This code constructs such a map following Sec. 3.1 of Driscoll & T.,
% Schwarz-Christoffel Mapping", 2002, evaluating integrals by a 
% stripped-down version of Laurie's double exponential quadrature code.
% For polygons with "crowding" one would need other quadrature methods.
% The map is made unique by z(N-2)=-1, z(N-1)=-i, z(N)=1.

% Define geometry & solve parameter problem:
  function schwarz_christoffel()
  p = [-1+1i -1-1i 1-1i 1 0 1i];              % vertices of P
  N = length(p);                              % number of vertices
  sides = angle(diff(p([N 1:N 1])))/pi;       % orientations of sides
  a = mod(1-diff(sides),2), b = a-1;          % angle parameters
  c = pi/2; h = 1/8;                          % compute quadrature data
  m1 = 6.6135/c; m2 = 3.6045/c;               %   -- these lines taken
  n1 = floor(m1/h); n2 = floor(m2/h);         %   from Laurie's q_de.m
  x = h*(-n1:n2); w = h*ones(1,length(x));   
  w = c*w.*cosh(c*x); x = sinh(c*x);        
  e2 = exp(x); e1 = 1./e2; s = e1+e2;
  x = (e2./s)'; w = 2*w./s./s;                % nodes and weights
  op = optimset('tolfun',1e-10);
  phi = fsolve(@scfun,zeros(1,N-3),op,p,b,N,x,w);
  t = cumsum(exp(cumsum([0 phi])));  
  z = exp(pi*1i*[t/t(end) -1/2 0]).'          % prevertices
  C = (p(2)-p(1))/scquad(z(1),z(2),z,b,N,x,w) % the constant C
  A = p(1) - C*scquad(0,z(1),z,b,N,x,w);      % the constant A

% Plot the map:
  figure, subplot(1,2,1), plot(exp(linspace(0,2i*pi)),'k')
  hold on, plot(z,'.'), axis(1.05*axis,'equal','off')
  for r = .1:.1:.9
    npts = round(20/(1-r)); zz = r*exp(linspace(0,2i*pi,npts));
    subplot(1,2,1), plot(zz,'b'), ww = 0*zz;
    for k = 1:npts, ww(k) = A + C*scquad(0,zz(k),z,b,N,x,w); end
    subplot(1,2,2), plot(ww,'r'), hold on
  end
  subplot(1,2,2), plot(p([1:N 1]),'k'), axis(1.05*axis,'equal','off')

  function I = scquad(z1,z2,z,b,N,x,w)        % evaluate SC integral
  zz = z1+x*(z2-z1); s = ones(size(zz));
  for k = 1:N, s = s.*(1-(1-1e-14)*zz/z(k)).^b(k); end
  I = (z2-z1)*w*s;

  function f = scfun(phi,p,b,N,x,w)           % S-C parameter problem
  t = cumsum(exp(cumsum([0 phi])));           % change of variables to
  z = exp(pi*1i*[t/t(end) -1/2 0]).';         %   eliminate constraints 
  C = (p(2)-p(1))/scquad(z(1),z(2),z,b,N,x,w);
  f = zeros(N-3,1);
  for k = 1:N-3
    s = C*scquad(z(k+1),z(k+2),z,b,N,x,w);
    f(k) = abs(s) - abs(p(k+2)-p(k+1));
  end
