% taylor_coeffs.m  Use FFT to compute Taylor coefficients of f(z)=exp(z)
%                  L. N. Trefethen 8/03
%
% If f(z) is analytic in the closed disk about z=0 of radius R>1,
% its jth Taylor coefficient is given by
%
% c_j  =  (2*pi*i)^(-1) INT z^-(j+1) f(z) dz
%
% where the integral is over the unit circle.  This integral can be
% evaluated with error O(R^(-N)) by the N-point trapezoid rule.  The
% Fast Fourier Transform does this for many values of j at once, giving
% an extraordinarily fast method for computing Taylor coefficients.
% This method was introduced by Lyness and Moler (1967) and Lyness and
% Sandee (1971) and surveyed by Henrici in his SIAM Review article
% "Fast Fourier methods in complex analysis" (1979) and in v. 3 of
% Applied and Computational Complex Analysis (Wiley, 1986).  This
% short code demonstrates the method for f(z)=exp(z).

  f = inline('exp(z)');           % exp(z) has recognizable coeffs
  N = 18;                         % often one would use, say, 64 points
  zz = exp(2i*pi*(0:N-1)'/N);     % roots of unity
  c = fft(f(zz))/N;               % FFT of f evaluated at roots of unity
  c = real(c);                    % c must be real by symmetry
  disp(['       computed    '...  % table headings
        '         exact     '])
  disp([c 1./gamma(1:N)'])        % results, approximate and exact
