% trapmap.m  Conformal map of a trapezoid to a rectangle
%            L. N. Trefethen 7/03
%
% Conformal maps of polygons can be represented by the Schwarz-
% Christoffel formula, which must then be implemented numerically.
% Another approach is to seek an efficient numerical representation
% directly.  This code follows that route to compute a conformal map
% w = f(z) of a trapezoid T to a rectangle R; it plots the map and
% prints the conformal module of T, i.e., the aspect ratio of R.
% f is analytic except at the lower-left corner of T, and the map
% is represented as a series of fractional powers at that point:
%
% w = f(z) = SUM_{k=1}^N c(k) z^((2/3)(2*k-1))
%
% We find the coefficients c(k) by solving a least-squares problem,
% a discretization of the conditions Im w' = 0 on the right boundary
% of T, Im w = 1 on the top boundary.

% Compute the coefficients:
  format long
  N = 30;                              % order of expansion
  np = 2*N;                            % no. of points on boundaries
  z1 = linspace(1,1+1i,np).';          % points on right boundary
  z2 = linspace(-1+1i,1+1i,np).';      % points on top boundary
  A1 = []; A2 = [];
  for k = 1:N
    s = (2/3)*(2*k-1);
    A1 = [A1 s*z1.^(s-1)];             % f' on right boundary
    A2 = [A2 z2.^s];                   % f on top boundary
  end
  A = [imag(A1); imag(A2)];            % matrix of sampled functions
  rhs = [zeros(np,1); ones(np,1)];     % right-hand side
  c = A\rhs                            % coefficients
  mu = sum(c)                          % conformal modulus
  
% Plot the map:
  subplot(2,1,1)
  plot([0 1 1+1i -1+1i 0],'r')
  hold on, axis equal, axis off
  subplot(2,1,2)
  plot([0 mu mu+1i 1i 0],'b')
  hold on, axis equal, axis off
  for y = .1:.1:.9
    z = linspace(y*(-1+1i),1+(1i*y),50).';
    subplot(2,1,1), plot(z,'r'), A = [];
    for k = 1:N, A = [A z.^((2/3)*(2*k-1))]; end
    subplot(2,1,2), plot(A*c,'b')
  end
  for x = -.9:.1:.9
    z = linspace(x-1i*min(x,0),x+1i,50).';
    subplot(2,1,1), plot(z,'r'), A = [];
    for k = 1:N, A = [A z.^((2/3)*(2*k-1))]; end
    subplot(2,1,2), plot(A*c,'b')
  end
