% pseudospectra.m  Compute pseudospectra of 60x60 Grcar matrix
%                  L. N. Trefethen 10/03
%
% This program computes pseudospectra by a method much faster than
% the obvious one of simply computing the SVD at many points.  The
% algorithm is due to Lui (SIAM J. Sci. Comp. 1997) and Trefethen
% (Acta Numerica 1999), and this code is adapted from psa.m in the
% latter paper.  The same algorithm is used by EigTool (Wright 2002).
 
% Define matrix and set up grid for contour plot:
  N = 60; A = gallery('grcar',N);                    % Grcar matrix
  npts = 40;                                         % grid resolution 
  x1 = -2.4; x2 = 4.4; y1 = -3.4; y2 = 3.4;          % grid limits
  x = x1:(x2-x1)/(npts-1):x2;
  y = y1:(y2-y1)/(npts-1):y2;
  [xx,yy] = meshgrid(x,y); zz = xx + 1i*yy;          % grid
 
% Compute Schur form and plot eigenvalues:
  [U,T] = schur(A);                                  % triangularize
  if isreal(A), [U,T] = rsf2csf(U,T); end
  T = triu(T); eigA = diag(T); hold off              % eigenvalues
  plot(real(eigA),imag(eigA),'.','markersize',10)
  hold on, axis([x1 x2 y1 y2]), axis square, drawnow
 
% Compute resolvent norms and plot contours:
  sigmin = Inf*ones(length(y),length(x)); I = eye(N);
  for i = 1:length(y)
    if isreal(A) & (y2==-y1) & (i>length(y)/2)       % exploit symmetry
      sigmin(i,:) = sigmin(length(y)+1-i,:);
    else
      for j = 1:length(x)
        z = zz(i,j); T1 = z*I-T; T2 = T1';           % shifted matrix
        sigold = 0; qold = zeros(N,1);
        beta = 0; H = [];
        q = randn(N,1)+1i*randn(N,1); q = q/norm(q); % initial vector
        for k = 1:99                                 % crude Lanczos loop
          v = T1\(T2\q) - beta*qold;                 % inverse Lanczos step
          alpha = real(q'*v); v = v - alpha*q;
          beta = norm(v); qold = q; q = v/beta;
          H(k+1,k) = beta; H(k,k+1) = beta;          % tridiagonal matrix
          H(k,k) = alpha;
          sig = max(eig(H(1:k,1:k)));
          if (abs(sigold/sig-1)<.001)|(sig<3&k>2)...
                break, end
          sigold = sig;
        end
        sigmin(i,j) = 1/sqrt(sig);                   % min. singular value
      end
    end
  end
  contour(x,y,log10(sigmin+1e-20),-8:-1)             % levels 1e-1 ... 1e-8
  colormap([0 0 0])                                  % make it monochrome
