% two_disks.m  Compute Green's function exterior to two disks
%              L. N. Trefethen 8/03
%
% Suppose we seek the real function g(z) that is harmonic (i.e., 
% satisfies Laplace's equation) exterior to |z-2|=1 and |z+2|=1 with
% g(z)~log|z| as |z| -> infty.  This function can be approximated
% with an error decreasing exponentially as N->inf by series
% 
% g(z) = .5*log|z-2| + .5*log|z+2|
%
%        + Re {SUM_{k=0}^N c(k)*[(2-z)^(-k)+(z+2)^(-k)]}.
%
% Given N, this code chooses good values for the coefficients c(k)
% by minimizing the sum of squares of the values of g(z) on equally-
% spaced points along the circles -- a linear least-squares problem.
% Symmetry is exploited to reduce the two circles to half of just
% one of them.  This method was used by Finn et al. in "Toplogical
% chaos in inviscid and viscous mixers", J. Fluid Mech. (2003).

  format long
  N = 20;                                 % no. of terms in expansion
  npts = 30;                              % no. of sample points
  z = 2 + exp((.5:npts-.5)'*1i*pi/npts);  % sample points on semicircle
  rhs = -real(log(2-z)+log(z+2))/2;       % right-hand side 
  A = [];
  for k = 0:N                             % set up least-squares matrix
    A(:,k+1) = (2-z).^(-k)+(z+2).^(-k);
  end
  A = real(A);                            % symmetry is imposed here
  c = A\rhs                               % solve least-squares problem
  value_at_0 = log(2) + 2.^(1:-1:1-N)*c   % print value at z=0

% Contour plot of the solution:
  x = linspace(-5,5,95); y = linspace(-4,4,75);
  [xx,yy] = meshgrid(x,y); zz = xx+1i*yy;
  gg = real(log(2-zz)+log(zz+2))/2;
  for k = 0:N
    gg = gg + c(k+1)*real((2-zz).^(-k)+(zz+2).^(-k));
  end
  gg(abs(2-zz)<1 | abs(zz+2)<1) = 0;
  levels = 1.1:.1:4;
  z = exp(pi*1i*(-50:50)'/50); z = [z+2 z-2];
  hold off, fill(real(z),imag(z),[.7 .7 1]), hold on
  plot(z,'-b','linewidth',2)
  contour(xx,yy,gg,log(levels))
  axis equal, axis([-5 5 -4 4]), colormap([0 0 0])
  set(gca,'xtick',-4:2:4,'ytick',-4:2:4)
